//**********************************************************************
// HttpGrabber
// ===========
//
// Grab data from an HTTP server
//
// Syntax: HttpGrabber <server> <url> [<output> [<username> <password>]]
//
// John Rennie
// jrennie@cix.compulink.co.uk
// 18th September 1997
//
// v1.1 20th June 1999
// Changed \n to \r\n in the request.  Added a Host: line to the
// request.  Without these changes the app fails with Freeserve.
//**********************************************************************

#ifndef STRICT
#define STRICT
#endif

#include <windows.h>
#include <winsock.h>
#include <stdio.h>
#include <RhsSock/RhsSock.h>


//**********************************************************************
// Function prototypes
// -------------------
//**********************************************************************

void Base64Encode(char* Input, char* Output);
BOOL strtoint(const char* s, int* i);
const char* GetLastErrorMessage(void);


//**********************************************************************
// Global variables
// ----------------
//**********************************************************************

#define SYNTAX \
"HttpGrabber <server> <path> [<output> [<username> <password>]]\n"

#define HELP \
"HttpGrabber <server> <path> [<output> [<username> <password>]]\n\n"          \
"Grab data from an http server\n"                                            \
"  e.g. HttpGrabber www.microsoft.com /\n\n"                                 \
"If <output> is omitted the output is written to the console.  To specify\n" \
"a username and password and still write output to the console set the\n"    \
"output to '.' e.g.\n"                                                       \
"  HttpGrabber www.microsoft.com / . myusername mypassword\n\n"              \
"To use a proxy just use the command in the form:\n"                         \
"  HttpGrabber myproxy.com http://www.whatever.com/\n"                       \
"that is use the proxy name as the server and use a full URL as the path.\n"


//**********************************************************************
// main
// ----
//**********************************************************************

int main(int argc, char* argv[])
{ int port, i;
  FILE* out;
  CRhsSocket skt;
  BOOL usepassword;
  char server[256], password[256], s[0x4000];

// Check the arguments

  if (argc == 2)
  { if (lstrcmp(argv[1], "-?") == 0 || lstrcmp(argv[1], "/?") == 0)
    { printf(HELP);
      return(0);
    }
  }

  if (argc < 3 || argc == 5)
  { printf(SYNTAX);
    return(2);
  }

// If required split the server into the server name and port

  port = 80;

  for (i = 0; argv[1][i] != ':' && argv[1][i] != '\0'; i++)
    server[i] = argv[1][i];
  server[i] = '\0';

  if (argv[1][i] == ':')
  { if (!strtoint(argv[1]+i+1, &port))
    { printf("\"%s\" is not a valid port number\n", argv[1]+i+1);
      return(2);
    }
  }

// Open the output stream

  out = stdout;

  if (argc > 3)
  { if (lstrcmp(argv[3], ".") != 0)
    { out = fopen(argv[3], "wb");

      if (!out)
      { printf("Cannot open output file \"%s\":\n%s", argv[3], GetLastErrorMessage());
        return(2);
      }
    }
  }

// If a username and password were supplied process them

  usepassword = FALSE;

  if (argc > 5)
  { wsprintf(s, "%s:%s", argv[4], argv[5]);
    Base64Encode(s, password);
    usepassword = TRUE;
  }

// OK open the connection

  fprintf(stderr, "Connecting to %s on port %i ...\n", server, port);

  skt.Startup();

  if (!skt.Connect(server, port))
  { printf("Cannot open %s\n", argv[1]);
    if (out != stdout)
    { fclose(out);
      DeleteFile(argv[3]);
    }
    return(2);
  }

  fprintf(stderr, "Connected\n\n");

// Send the header

  skt.printf("GET %s HTTP/1.0\r\n"
             "Host: %s\r\n"
             "User-Agent: RawHttpGrabber\r\n"
             "Accept: */*\r\n",
             argv[2], argv[1]
            );

  if (usepassword)
    skt.printf("Authorization: Basic %s\r\n", password);

  skt.printf("\r\n");

// Read the response

  while ((i = skt.read(s, 1, 0x4000)) > 0)
  { fwrite(s, 1, i, out);

    if (out != stdout)
      fprintf(stderr, ".");
  }

  fprintf(stderr, "\n***Done***\n");

// All done, close the connection

  skt.CloseSocket();
  skt.Cleanup();

  if (out != stdout)
    fclose(out);

// All done

  return(0);
}


//**********************************************************************
// Base64Encode
// ============
// Encode a string as Base64
//**********************************************************************

static char Base64EncodeMap[64] =
{'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
 'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
 'w', 'x', 'y', 'z', '0', '1', '2', '3',
 '4', '5', '6', '7', '8', '9', '+', '/'
};

void Base64Encode(char* Input, char* Output)
{ int i, j;
  unsigned u, v;

  i = j = 0;

  while (Input[i] != '\0')
  { u = (unsigned) Input[i++];
    Output[j++] = Base64EncodeMap[(u >> 2) & 0x3F];
    u = (u & 0x03) << 4;

    if (Input[i] == '\0')
    { Output[j++] = Base64EncodeMap[u];
      Output[j++] = '=';
      Output[j++] = '=';
    }
    else
    { v = (unsigned) Input[i++];
      u = u + ((v >> 4) & 0x0F);
      Output[j++] = Base64EncodeMap[u];
      u = (v & 0x0F) << 2;

      if (Input[i] == '\0')
      { Output[j++] = Base64EncodeMap[u];
        Output[j++] = '=';
      }
      else
      { v = (unsigned) Input[i++];
        u = u + ((v >> 6) & 0x03);
        Output[j++] = Base64EncodeMap[u];
        Output[j++] = Base64EncodeMap[v & 0x3F];
      }
    }
  }

  Output[j] = '\0';
}


//**********************************************************************
// strtoint
// --------
// Convert string to int
//**********************************************************************

BOOL strtoint(const char* s, int* i)
{ const char* start;
  char* end;

  for (start = s; *start == ' ' || *start == '\t'; start++);
  *i = (int) strtol(start, &end, 10);

  return(end > start ? TRUE : FALSE);
}


//**********************************************************************
// GetLastErrorMessage
// -------------------
//**********************************************************************

const char* GetLastErrorMessage(void)
{ DWORD d;
  static char errmsg[1024];

  d = GetLastError();

  strcpy(errmsg, "<unknown error>");
  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, d, 0, errmsg, 1023, NULL);
  return(errmsg);
}


